/*
 * Copyright 2013 Canonical Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef NETWORK_H
#define NETWORK_H

#include <QObject>
#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkReply>
#include <QHash>
#include "application.h"

#define X_CLICK_TOKEN "X-Click-Token"

namespace ClickPlugin {

class Network : public QObject
{
    Q_OBJECT
public:
    explicit Network(QObject *parent = 0);

    void checkForNewVersions(QHash<QString, Application*> &apps);
    void getResourceUrl(const QString& packagename);
    void getClickToken(Application* app, const QString& url, const QString& authHeader);

signals:
    void updatesFound();
    void updatesNotFound();
    void errorOccurred();
    void downloadUrlFound(QString packagename, QString url);
    void clickTokenObtained(Application* app, const QString& clickToken);

private slots:
    void onReply(QNetworkReply*);

private:
    QNetworkAccessManager* _nam;
    QNetworkRequest* _request;
    QHash<QString, Application*> _apps;
};

}

#endif // NETWORK_H
